/* ================================================================================================================================
  This script was created by SkyTruth to identify active mountaintop removal and other surface coal mining activity. This script 
  requires an inputImageCollection, generated with the greenestCompositesToAssets_allBand_allIndex script. This script is an 
  adaptation of the work conducted with Andrew Pericak, but now creates more robust vectors and adds information from all available
  bands and all calculated indices.

  While annual imagery is available from 1984-2022, both 1984 and 2022 are used only for data cleaning purposes.
  
  This code is updated from the code available in the 2021_MTM_Footprint folder, the C01/T1 Landsat Collections are now deprecated,
  this code makes use of C02/T1 collections. It also adds Landsat 9 to the collection. Outputs are writted to the same collections
  as those used in 2021_MTM_Footprint, although this is only to add data from 2022 onwards.
-----------------------------------------------------------------------------------------------------------------------------------
	The methodology of the MTM data produced for PLOS One relied on the now deprecated Pre-Collection Landsat data. The USGS 
	switched to the Collection format around May 2017. This change has prompted a shift to use USGS Landsat Collection 1 Tier 1 and 
	Real-Time Raw Scenes, and is reflected here. Read about the original study here: 
	https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0197758
================================================================================================================================ */


/* ============================================ IMPORT COMPOSITE IMAGE COLLECTIONS ============================================= */
var greenestComposites = ee.ImageCollection('users/skytruth-data/MTR/TOA_GPC');
var toa_gpc_thresholds_1 = ee.ImageCollection('users/skytruth-data/MTR/annualThresholdImages_2019-Mask');
var toa_gpc_thresholds_2 = ee.ImageCollection('users/skytruth-data/MTR/annualThresholdImages_2021-Mask');
var toa_gpc_thresholds = toa_gpc_thresholds_1.merge(toa_gpc_thresholds_2);
var toa_vectorDescription = '_activeMining_footprintOnly_toaGPC_5072';
var toa_imageDescription = '_activeMining_toaGPC_5072';
var toa_cumulativeImageDescription = "CumulativeMineArea_toaGPC_1985-2021_5072";

var sr_rma_med_Composites = ee.ImageCollection('users/skytruth-data/MTR/SR_RMA_Harmonized_medianComposite').select(['B','G','R','NIR','NDVI']);
var rma_med_thresholds_1 = ee.ImageCollection('users/skytruth-data/MTR/annualThresholdImages_SR-RMA-MED_2019-Mask');
var rma_med_thresholds_2 = ee.ImageCollection('users/skytruth-data/MTR/annualThresholdImages_SR-RMA-MED_2021-Mask');
var rma_med_thresholds = rma_med_thresholds_1.merge(rma_med_thresholds_2);
var rma_vectorDescription = '_activeMining_footprintOnly_rmaMED_5072';
var rma_imageDescription = '_activeMining_rmaMED_5072';
var rma_cumulativeImageDescription = "CumulativeMineArea_rmaMED_1985-2020";


var composite = greenestComposites;
var thresholds = toa_gpc_thresholds;
var vectorDescription = toa_vectorDescription;
var imageDescription = toa_imageDescription;
var cumulativeDescription = toa_cumulativeImageDescription;
var crs = "EPSG:5072";
/* ================================================ IMPORT STUDY AREA =========================================================== */
// https://www.google.com/fusiontables/DataSource?docid=1Lphn5PR9YbneoY4sPkKGMUOJcurihIcCx0J82h7U
var studyArea = ee.FeatureCollection('users/skytruth-data/Plos_MTM_Fusion_Table_Backup/plosScriptFusionTableBackup/studyArea');

/* ================================================ ESTABLISH EXPORT AREAS ==================================================== */
//// BY COUNTY
var fips_codes = [21013,21019,21025,21043,21051,21053,21063,21065,21071,21089,21095,21109,21115,21119,21121,21125,21127,21129,21131,
                  21133,21135,21147,21153,21159,21165,21175,21189,21193,21195,21197,21199,21203,21205,21231,21235,21237,47001,47013,
                  47025,47035,47049,47129,47133,47137,47141,47145,47151,51027,51051,51105,51167,51169,51185,51195,51720,54005,54011,
                  54015,54019,54025,54039,54043,54045,54047,54053,54055,54059,54067,54075,54079,54081,54089,54099,54101,54109];

var allCounties = ee.FeatureCollection('users/skytruth-data/Plos_MTM_Fusion_Table_Backup/plosScriptFusionTableBackup/allCounties');
var features = allCounties.filter(ee.Filter.inList('FIPS', fips_codes));

// This creates an image where each pixel is labeled with its FIPS code
var countyImg = features.reduceToImage({
  properties: ["FIPS"],
  reducer: ee.Reducer.first()
}).rename("FIPS");

/* =================================================== IMPORT MASK ============================================================= */
// var mask_input_60m_2019 = ee.Image('users/skytruth-data/MTR/2019_Input-Mask_CT_4326').unmask();
var mask_input_60m_2021 = ee.Image('users/skytruth-data/MTR/2021_Input-Mask_CT_4326').unmask();
// Roads, water bodies, urban areas, etc., buffered 60 m

var miningPermits = ee.Image('users/andrewpericak/allMinePermits');
// All surface mining permits, buffered 1000 m. Get the link here: https://drive.google.com/file/d/0B_PTuMKVy7beSWdZUkJIS3hneW8/view

var miningPermits_noBuffer = ee.Image('users/andrewpericak/allMinePermits_noBuffer');

// All surface mining permits, without any buffer. This excludes from the 60 m mask (mask_input_60m_2015) any areas within the the 
// mine permit boundaries. Since the 60 m mask is "inversed" (i.e., 0 means keep and 1 means eventually mask), this line sets the 
// mine permit areas (labeled as 1) to 0.

var mask_input_excludeMines = mask_input_60m_2021.where(miningPermits_noBuffer.eq(1), 0);

/* =================================================== MINE ANALYSIS =========================================================== */
// INITIAL MINE THRESHOLDING
// Create a list of yearly threshold images, and a list of years associated with those images, for image selection within the loop
var threshImgList = thresholds.sort("year").toList(100);
var threshImgYrList = ee.List(thresholds.aggregate_array("year")).sort();


// RAW MINING
// This initially compares the NDVI at each pixel to the given threshold. The resulting images are 1 = mine and 0 = non-mine.
var rawMining = ee.ImageCollection(composite.map(function(image){
  var year = image.get("year");
  var mineCandidate = image.select("NDVI").rename('mineCandidate');
  var ndvi = image.select('NDVI');
 
  // This pulls the specific threshold image for the given year
  var index = ee.Number(threshImgYrList.indexOf(year));
  var threshold = ee.Image(threshImgList.get(index));
  
  // This compares the NDVI per pixel to the NDVI threshold 
  var lowNDVI = mineCandidate.lte(threshold);
  return lowNDVI.set({"year": year});
}));

// NULL-VALUE CLEANIING
// This is the first step of the null-value cleaning (i.e., where there are no pixel values for that year due to cloud cover and a 
// lack of raw imagery.) This function first creates a binary image where 1 = values that were null in the rawMining image, and 
// where 0 = everything else.
var nullCleaning1 = ee.ImageCollection(rawMining.map(function(image){
  var year = image.get("year");
  var unmasked = image.lt(2).unmask().not().clip(studyArea);
  return unmasked.set({"year": year});
}));

// Create dummy images so the null cleaning will work; essentially for 1983 and 2017 (the years immediately before and after the 
// years for which we have Landsat scenes), we need images of value 0 so that the nullCleaning2 function below actually works. 

var dummy1983 = ee.Image(0).rename("NDVI").set({"year": 1983});
var dummy2023 = ee.Image(0).rename("NDVI").set({"year": 2023});

var rawMining2 = ee.ImageCollection(rawMining.merge(ee.ImageCollection([dummy1983,dummy2023])));

// Create two lists in order to help choose the immediate prior and future images for a given year. The first is a list of each 
// image in the rawMining2 ImageCollection, sorted by year; the second is just a list of each year present in our dataset. These 
// lists, used below, make running the code much more efficient than using filterMetadata() or anything like that. Similar lists 
// seen below are achieving the same effect.
var rm2List = rawMining2.sort("year").toList(100);
var rm2YrList = ee.List(rawMining2.aggregate_array("year")).sort();

// This is the second step of the null-value cleaning. For each year, pull the raw mining images for the years immediately prior 
// and future, where in those images 1 = mine and 0 = non-mine. Add those three images together; areas where the sum is 3 indicate 
// that the null pixel is likely a mine, because that pixel was a mine in the prior and future years.
var nullCleaning2 = ee.ImageCollection(nullCleaning1.map(function(image){
  var year = ee.Number(image.get("year"));

  var rm2Index = ee.Number(rm2YrList.indexOf(year));  // Get the index of the year that is being looked at
  var priorIndex = rm2Index.subtract(1);              // The previous year index
  var futureIndex = rm2Index.add(1);                  // The future year index
  var imgPrevious = ee.Image(rm2List.get(priorIndex)).unmask();
  var imgNext = ee.Image(rm2List.get(futureIndex)).unmask();
      // Since the indices are the same for rm2List and rm2YrList, essentially use the index of the year 
      // to select the corresponding image, which has the same index.

  var summation = image.add(imgPrevious).add(imgNext);
  var potentialMine = summation.eq(3);
      // In other words, if a null in current year (1) and a mine in past and future years (also 1), the 
      // pixel in the current year will sum to 3
  return potentialMine.set({"year": year});
}));

var nc2List = nullCleaning2.sort("year").toList(100);
var nc2YrList = ee.List(nullCleaning2.aggregate_array("year")).sort();

// This is the third step of the null-value cleaning. Use the results of the previous operation to turn any null values in the 
// rawMining imagery into a value of 1 if they were also a value of 1 from the nullCleaning2 imagery.

var nullCleaning3 = ee.ImageCollection(rawMining.map(function(image){
  var year = image.get("year");
  var nc2Index = ee.Number(nc2YrList.indexOf(year));
  var cleaningImg = ee.Image(nc2List.get(nc2Index));
  var updatedRaw = image.unmask().add(cleaningImg);
  return updatedRaw.set({"year": year});
}));

// NOISE CLEANING 1
// Now we have noise cleaning, to eliminate pixels that go from unmined->mine-> unmined, since the mine pixel is likely incorrect. 
// Again we need dummy years for years without LS coverage, except this time the dummy images have values of 1 because otherwise 
// all the pixels near these years would get removed.
var dummy1983a = ee.Image(1).rename("NDVI").set({"year": 1983});
var dummy2023a = ee.Image(1).rename("NDVI").set({"year": 2023});

var rawMining3 = ee.ImageCollection(nullCleaning3.merge(ee.ImageCollection([dummy1983a,dummy2023a])));
var rm3List = rawMining3.sort("year").toList(100); // See explanation above
var rm3YrList = ee.List(rawMining3.aggregate_array("year")).sort();

var noiseCleaning1 = ee.ImageCollection(nullCleaning3.map(function(image){
  var year = ee.Number(image.get("year"));
  var rm3Index = ee.Number(rm3YrList.indexOf(year));
  var priorIndex = rm3Index.subtract(1);
  var futureIndex = rm3Index.add(1);
  var imgPrevious = ee.Image(rm3List.get(priorIndex));
  var imgNext = ee.Image(rm3List.get(futureIndex));
  // Relabel images so that pixels that are mine in current year but not mine
  // in previous/next years are labeled 111 when summed
  var relabelPrevious = imgPrevious.remap([0,1],[100,-10]);
  var relabelNext = imgNext.remap([0,1],[10,-10]);
  var summation = image.add(relabelPrevious).add(relabelNext);
  var potentialNoise = summation.eq(111);
      // Mine in current year = 1; non-mine in past year = 100; non-mine in 
      // future year = 10; therefore we want sum of 111
  return image.where(potentialNoise.eq(1),0).set({"year":year});
}));

// NOISE CLEANING 2
// And more noise cleaning, now to eliminate pixels that go from mined->unmined-> mined, since the unmined pixel is likely 
// incorrect. We need the 0-value dummy images from above.
var rawMining4 = ee.ImageCollection(noiseCleaning1.merge(ee.ImageCollection([dummy1983,dummy2023])));
var rm4List = rawMining4.sort("year").toList(100); // See explanation above
var rm4YrList = ee.List(rawMining4.aggregate_array("year")).sort();

var noiseCleaning2 = ee.ImageCollection(noiseCleaning1.map(function(image){
  var year = ee.Number(image.get("year"));
  var rm4Index = ee.Number(rm4YrList.indexOf(year));
  var priorIndex = rm4Index.subtract(1);
  var futureIndex = rm4Index.add(1);
  var imgPrevious = ee.Image(rm4List.get(priorIndex));
  var imgNext = ee.Image(rm4List.get(futureIndex));
  // Relabel images so that pixels that are mine in current year but not mine
  // in previous/next years are labeled 111 when summed
  var relabelPrevious = imgPrevious.remap([0,1],[-10,900]);
  var relabelNext = imgNext.remap([0,1],[-10,90]);
  var summation = image.add(relabelPrevious).add(relabelNext);
  var potentialNoise = summation.eq(990);
      // Mine in current year = 1; non-mine in past year = 100; non-mine in 
      // future year = 10; therefore we want sum of 111
  return image.where(potentialNoise.eq(1),1).set({"year":year});
}));


// FINAL CLEANING AND OUTPUT
// This final function performs other cleaning (mask, erosion/dilation, mine permit boundaries) to further clean up the mining 
// dataset.
var mining = ee.ImageCollection(noiseCleaning2.map(function(image){
  var year = ee.Number(image.get("year"));
// Create binary image containing the intersection between the LowNDVI and anywhere the inverted mask is 0
  var mines = image.select('mineCandidate').and(mask_input_excludeMines.eq(0));
  
  // Mask mine layer by itself; label with specific year (for viz)
  var minesMasked = mines.updateMask(mines).multiply(year).rename('year').toInt();
  
  // Remove small, noisy pixel areas (remember, this is dependent on zoom level)
  var smallAreaMask = minesMasked.connectedPixelCount().gte(10);
  var noSmall = minesMasked.updateMask(smallAreaMask);
  
  // Compute area per pixel
  var area = ee.Image.pixelArea().multiply(noSmall).divide(year).rename('area').toFloat();
  
  return image.addBands(noSmall).addBands(area).addBands(countyImg);
}));

/* ================================================ ACCURACY ASSESSMENT ======================================================== */
// This will add properties to each yearly image, reporting overall accuracy (accuracy), kappa coefficient (kappa), user's error 
// (user), and producer's error (producer)
var accuracy = mining.select("year").map(function(image) {
  var year = image.get('year');
  image = image.gt(0).unmask();
  // The next line is a Fusion Table containing all manually-classified points as one table. This table must contain a binary 
  //column called CLASS where value 0 = non-mine and value 1 = mine.
  
  var points = ee.FeatureCollection('users/skytruth-data/MTR/Classification_Points/accuracyAssessmentPoints_1984-2022_created_20221201')
    .filterMetadata("YEAR","equals",year);
  var output = image.sampleRegions(points, ['CLASS'], 30);
  var error_matrix = output.errorMatrix('CLASS', 'year');
  return image.set({"accuracy": error_matrix.accuracy(), 
    "kappa": error_matrix.kappa(),
    "user": error_matrix.consumersAccuracy().toList().flatten(), 
    "producer": error_matrix.producersAccuracy().toList().flatten(), 
    "year": year});
});

// This turns the accuracy assessement values into blank geometries, so that they can be exported into tables.
var accuracyCollection = ee.FeatureCollection(accuracy.map(function(image){
    var dict = ee.Dictionary({});
    var year = image.get("year");
    var accuracy = image.get("accuracy");
    var kappa = image.get("kappa");
    var user = image.get("user");
    var producer = image.get("producer");
    
    dict = ee.Dictionary(ee.Dictionary(ee.Dictionary(ee.Dictionary(dict
      .set("accuracy",accuracy))
      .set("kappa",kappa))
      .set("user",user))
      .set("producer",producer))
      .set("year",year);
    return ee.Feature(ee.Geometry.Point(0,0),dict);
}));

Export.table.toDrive({
  collection: accuracyCollection,
  description: "accuracyAssessmentResults_1984-2022",
  folder: 'MTM_Output_Data',
  fileFormat: "csv"
});

/* ================================================================================================================================
                                          SUMMARIZE MINING AREA PER FEATURE PER YEAR                      
================================================================================================================================ */
// This creates a table listing the area of active mining per year, per subregion noted above (either county or rectangle).

var getFeatures = function(feature) {
  var fips = feature.get("FIPS");
  return mining.map(function(image) {
    var yearlyArea = image.select('area').reduceRegion({
      reducer: 'sum', 
      geometry: feature.geometry(), 
      scale: 30,
      crs: "EPSG:5072",
      maxPixels: 1e10
    });
    return image.set(yearlyArea).set({"FIPS": fips});
  });
};
var miningArea = features.map(getFeatures).flatten();

// This will export the table to your Google Drive
Export.table.toDrive({
  collection: miningArea,
  // description: "yearlyActiveMiningArea_RMA",
  description: "yearlyActiveMiningArea",
  folder: 'MTM_Output_Data',
  fileFormat: "csv"
});

/* ================================================================================================================================
                                                        EXPORT TO VECTORS                      
================================================================================================================================ */

// This will export the active mining areas for a given year as polygons, where each polygon will be labeled with its EE-calcuated
// area and the year

// Set the year of export here
var vectorYr = 2021; // 1985-2022

var compositeYear = composite.filterMetadata('year','equals',vectorYr).first();
Map.addLayer(compositeYear,{bands:['R','G','B'],min:0,max:3000},vectorYr+' Composite');
Map.addLayer(mining.filterMetadata("year","equals",vectorYr),{palette:['b80000'] ,bands:["year"], min:1983, max:2020}, vectorYr+" mining");


/* ================================================== CREATE STATS IMAGES ====================================================== */

// Area Sum
var vectorImg = ee.Image(mining.filterMetadata('year','equals',vectorYr).first()).select('year','area');

// CREATE THE OUTPUT VECTOR
var vectors = vectorImg.reduceToVectors({reducer: ee.Reducer.sum(),geometry: features.geometry(),scale: 30,crs: crs,
                                        labelProperty: 'year', maxPixels: 1e10}).set({'year':vectorYr});

Export.table.toDrive({
  collection: vectors,
  description: vectorYr+vectorDescription,
  folder: 'MTM_Output_Data',
  fileFormat: "geojson"
});

/* ================================================================================================================================
                                                        EXPORT IMAGES                      
================================================================================================================================ */

/* =============================================== Yearly Mining Footprint ===================================================== */
var yearlyMining_raster = ee.Image(mining.filterMetadata("year","equals",vectorYr).first()).select("year").unmask();

Export.image.toDrive({
  image: yearlyMining_raster,
  description: vectorYr+imageDescription,
  folder: 'MTM_Output_Data',
  region: studyArea.geometry(),
  scale: 30,
  crs: crs,
  maxPixels: 1e10
});

// /* ==============================  Yearly Mining Footprint with Data ================================== */
// var yearMiningData = ee.Image(mining.filterMetadata("year","equals",vectorYr).first())
//   .select(['year','area'])
//   .cast({'year':'float','area':'float','FIPS':'float'});
// // Export.image.toDrive({
// //   image: yearMiningData,
// //   description: "mining_data_"+vectorYr,
// //   folder: 'MTM_Output_Data',
// //   region: studyArea.geometry(),
// //   scale: 30,
// //   crs: "EPSG:5072",
// //   maxPixels: 1e10
// // });
/* ================================ Cumulative Mining Footprint ======================================= */
var cumulativeArea = mining.filterMetadata("year","greater_than",1984)
                          .filterMetadata("year","less_than",2022)
                          .reduce(ee.Reducer.anyNonZero());

// This will give the number in square meters (print the output)
var TCA_pixelArea = cumulativeArea.select("mining_any").multiply(ee.Image.pixelArea());

var TCA_reducer = TCA_pixelArea.reduceRegion({
  reducer: ee.Reducer.sum(),
  geometry: features.geometry(),
  scale: 30,
  crs: crs,
  maxPixels:1e10
});

Export.image.toDrive({
  image: cumulativeArea,
  description: cumulativeDescription,
  folder: 'MTM_Output_Data',
  region: studyArea.geometry(),
  scale: 30,
  crs: crs,
  maxPixels: 1e10
});
var palette = ["ffffcc","ffeda0","fed976","feb24c","fd8d3c","fc4e2a","e31a3c","bd0026","800026"];
// Full set of mining
// Map.addLayer(mining.filterMetadata("year","greater_than",1983), {bands:["year"], min:1983, max:2021,palette:palette}, "symbolized mining");
Map.addLayer(mining, {bands:["year"], min:1983, max:2020,palette:palette}, "mining");
// Map.addLayer(joined_8_fin,{color:'blue'},'Final')
// Show accuracy assessment values
print(accuracy);