/* ================================================================================================================================
  This script was created by SkyTruth to create annual greenest-pixel (maximum NDVI) composites and to save those composites as an 
  Earth Engine ImageCollection. This collection is used as input data for the annual coal surface mining extent analysis.
  This code differs from others written after the publication of the Plos paper because it preserves all band data and creates 
  multiple indexes.
  
  This code is updated from the code available in the 2021_MTM_Footprint folder, the C01/T1 Landsat Collections are now deprecated,
  this code makes use of C02/T1 collections. It also adds Landsat 9 to the collection. Outputs are writted to the same collections
  as those used in 2021_MTM_Footprint, although this is only to add data from 2022 onwards.
-----------------------------------------------------------------------------------------------------------------------------------
	The methodology of the MTM data produced for PLOS One relied on the now deprecated Pre-Collection Landsat data. The USGS 
	Switched to the Collection format around May 2017. This change has prompted a shift to use USGS Landsat Collection 1 Tier 1 and 
	Real-Time Raw Scenes, and is reflected here. Read about the original study here: 
	https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0197758
================================================================================================================================ */

// Import the Study Area
var studyArea = ee.FeatureCollection('users/skytruth-data/Plos_MTM_Fusion_Table_Backup/plosScriptFusionTableBackup/studyArea');
var exportbounds = studyArea.geometry().bounds();

// Import Image Collections
var lt4_raw = ee.ImageCollection('LANDSAT/LT04/C02/T1').select('B1','B2','B3','B4','B5','B6','B7');
var lt5_raw = ee.ImageCollection('LANDSAT/LT05/C02/T1').select('B1','B2','B3','B4','B5','B6','B7');
var le7_raw = ee.ImageCollection('LANDSAT/LE07/C02/T1').select('B1','B2','B3','B4','B5','B6_VCID_1','B6_VCID_2','B7','B8');
var lc8_raw = ee.ImageCollection('LANDSAT/LC08/C02/T1').select('B1','B2','B3','B4','B5','B6','B7','B8','B9','B10','B11');
var lc9_raw = ee.ImageCollection('LANDSAT/LC09/C02/T1').select('B1','B2','B3','B4','B5','B6','B7','B8','B9','B10','B11');

var cleaner = function(outlierValue) {
  return function(image){
      var min_mask = image.mask().reduce(ee.Reducer.min());
      var sat_mask = image.reduce(ee.Reducer.max()).lt(outlierValue);
      var new_mask = min_mask.min(sat_mask).focal_min(3);
      return ee.Algorithms.Landsat.TOA(image).updateMask(new_mask);
  };
};

var maskClouds = function(image) {
  var scored = ee.Algorithms.Landsat.simpleCloudScore(image);
  return image.updateMask(scored.select(['cloud']).lt(20));
};

// Create a mask image which will be used to create matching thermal bangs
var blank = ee.Image().mask();
var newMaskBand = ee.Image().updateMask(blank).toFloat();

var lt4Collection = lt4_raw
  .filterBounds(studyArea)
  .filterDate('1984-01-01', '1993-12-31')
  .map(cleaner(250))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B3'),NIR:image.select('B4')}).rename('NDVI'))
    
  .select(['B1','B2','B3','B4','NDVI'])
  .rename(['B','G','R','NIR','NDVI']);
  });

var lt5Collection = lt5_raw
  .filterBounds(studyArea)
  .filterDate('1984-01-01', '2011-12-31')
  .map(cleaner(250))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B3'),NIR:image.select('B4')}).rename('NDVI'))
    
  .select(['B1','B2','B3','B4','NDVI',])
  .rename(['B','G','R','NIR','NDVI']);
  });

var le7Collection = le7_raw
  .filterBounds(studyArea)
  .filterDate('1999-01-01', Date.now())
  .filter(ee.Filter.date("2012-08-03","2012-08-04").not()) // This line removes a specific image that has too many errors
  .map(cleaner(250))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B3'),NIR:image.select('B4')}).rename('NDVI'))
    
  .select(['B1','B2','B3','B4','NDVI'])
  .rename(['B','G','R','NIR','NDVI']);
  });
  
var lc8Collection = lc8_raw
  .filterBounds(studyArea)
  .filterDate('2013-01-01', Date.now())
  .map(cleaner(64256))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B4'),NIR:image.select('B5')}).rename('NDVI'))
    
  .select(['B2','B3','B4','B5','NDVI'])
  .rename(['B','G','R','NIR','NDVI']);
  });

var lc9Collection = lc9_raw
  .filterBounds(studyArea)
  .filterDate('2021-10-31', Date.now())
  .map(cleaner(64256))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B4'),NIR:image.select('B5')}).rename('NDVI'))
    
  .select(['B2','B3','B4','B5','NDVI'])
  .rename(['B','G','R','NIR','NDVI']);
  });

var lsCollection = ee.ImageCollection((lt4Collection)
    .merge(lt5Collection)
    .merge(le7Collection)
    .merge(lc8Collection)
    .merge(lc9Collection));

/*------------------------ EXPORTING YEARLY COMPOSITES -----------------------*/
var year = 2022; // 1984-2022

// Make the annual composite for the year noted above
var composite = lsCollection
      .filter(ee.Filter.calendarRange(year, year, 'year'))
      
      .filter(ee.Filter.calendarRange(5, 9, 'month'))
        // The above line is for restricting imagery to summer months. Using the
        // same date range as that defined in PLOS One
      .qualityMosaic('NDVI')
      .set({"year": year}); // This adds the year as a property to the image

// Export to GEE asset
var description = "composite_"+year; // Be careful here with multi-year MSS
Map.addLayer(composite,{bands:['R','G','B'],min:0,max:0.3}, year+' Composite',false);

Export.image.toAsset({
  image: composite,
  description: description,
  assetId: ("users/skytruth-data/MTR/TOA_GPC/"+description),
  region: exportbounds,
  scale: 30,
  maxPixels: 1e10
});
